@@ -11,19 +11,19 @@ A_data_bad_non_unique <- c(A_data, 2)
 test_that("Failures based on input types", {
   
   expect_error(bayesTest(A_data, B_data, priors = c('alpha' = 5, 'beta' = 3, "jumanji" = 6), distribution = 'bernoulliC'),
-               "Incorrect length of priors. Expecting an argument for alpha and beta ONLY.")
+               "Incorrect number of priors for supplied distribution.")
   
   expect_error(bayesTest(c(A_data, -3), B_data, priors = c('alpha' = 5, 'beta' = 3), distribution = 'bernoulliC'),
-               "Data input is incorrect. Data can only contain 0's and 1's. See help docs for more info.", fixed = TRUE)
+               "A_data %in% c(0, 1) are not all TRUE", fixed = TRUE)
   
   expect_error(bayesTest(A_data, B_data, priors = c('alpha' = 1, 'fergalicious' = 1), distribution = 'bernoulliC'),
-               "Arguments don't match requirement for alpha and beta. Check names.")
+               "Misnamed priors provided for supplied distribution.")
   
   expect_error(bayesTest(A_data, B_data, priors = c('alpha' = 'male', 'beta' = 1), distribution = 'bernoulliC'),
-               "alpha and/or beta are not numeric!")
+               "One or more priors aren't numeric.")
   
   expect_error(bayesTest(A_data, B_data, priors = c('alpha' = 0, 'beta' = 1), distribution = 'bernoulliC'),
-               "alpha and beta are parameters of the Beta Distribution and should be strictly > 0.")
+               "alpha > 0 is not TRUE")
 
 })
 
@@ -33,6 +33,6 @@ test_that("Success", {
   
   expect_is(successfulTest, "bayesTestClosed")
   
-  expect_output(str(successfulTest), "List of 3") # outer
-  
+  expect_output(str(successfulTest), "List of 2") # outer
+  expect_output(str(successfulTest), "List of 5") # inputs
 })
